@@ -61,8 +61,8 @@
 #include <Foundation/Foundation.h>
 
 #include <QtCore/qdatetime.h>
-
 #include <QtCore/qurl.h>
+
 #include <QtCore/qdebug.h>
 
 QT_USE_NAMESPACE
@@ -121,15 +121,20 @@ QT_USE_NAMESPACE
                                                      name:QTMovieVolumeDidChangeNotification
                                                    object:m_movie];
 
-        [[NSNotificationCenter defaultCenter] addObserver:self
-                                                 selector:@selector(processNaturalSizeChange:)
-                                                     name:
-#if defined(MAC_OS_X_VERSION_10_6) && (MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_6)
-                                                          QTMovieNaturalSizeDidChangeNotification
-#else
-                                                          QTMovieEditedNotification
-#endif
-                                                   object:m_movie];
+        if (QSysInfo::MacintoshVersion >= QSysInfo::MV_10_6) {
+            [[NSNotificationCenter defaultCenter] addObserver:self
+                                                     selector:@selector(processNaturalSizeChange:)
+                                                         name:@"QTMovieNaturalSizeDidChangeNotification"
+                                                       object:m_movie];
+
+        }
+        else {
+            [[NSNotificationCenter defaultCenter] addObserver:self
+                                                     selector:@selector(processNaturalSizeChange:)
+                                                         name:QTMovieEditedNotification
+                                                       object:m_movie];
+        }
+
         [movie retain];
     }
 }
@@ -173,6 +178,7 @@ QT7PlayerSession::QT7PlayerSession(QObject *parent)
    , m_mediaStream(0)
    , m_videoOutput(0)
    , m_muted(false)
+   , m_tryingAsync(false)
    , m_volume(100)
    , m_rate(1.0)
    , m_duration(0)
@@ -213,8 +219,6 @@ qint64 QT7PlayerSession::position() const
     if (!m_QTMovie || m_state == QMediaPlayer::PausedState)
         return m_currentTime;
 
-    AutoReleasePool pool;
-
     QTTime qtTime = [(QTMovie*)m_QTMovie currentTime];
     quint64 t = static_cast<quint64>(float(qtTime.timeValue) / float(qtTime.timeScale) * 1000.0f);
     m_currentTime = t;
@@ -227,8 +231,6 @@ qint64 QT7PlayerSession::duration() const
     if (!m_QTMovie)
         return 0;
 
-    AutoReleasePool pool;
-
     QTTime qtTime = [(QTMovie*)m_QTMovie duration];
 
     return static_cast<quint64>(float(qtTime.timeValue) / float(qtTime.timeScale) * 1000.0f);
@@ -276,9 +278,10 @@ void QT7PlayerSession::setPlaybackRate(qreal rate)
 
     m_rate = rate;
 
-    if (m_QTMovie && m_state == QMediaPlayer::PlayingState) {        
+    if (m_QTMovie != 0 && m_state == QMediaPlayer::PlayingState) {
+        AutoReleasePool pool;
         float preferredRate = [[(QTMovie*)m_QTMovie attributeForKey:@"QTMoviePreferredRateAttribute"] floatValue];
-        [(QTMovie*)m_QTMovie setRate:preferredRate*m_rate];
+        [(QTMovie*)m_QTMovie setRate:preferredRate * m_rate];
     }
 }
 
@@ -287,8 +290,6 @@ void QT7PlayerSession::setPosition(qint64 pos)
     if ( !isSeekable() || pos == position())
         return;
 
-    AutoReleasePool pool;
-
     pos = qMin(pos, duration());
 
     QTTime newQTTime = [(QTMovie*)m_QTMovie currentTime];
@@ -298,23 +299,31 @@ void QT7PlayerSession::setPosition(qint64 pos)
 
 void QT7PlayerSession::play()
 {
+    if (m_state == QMediaPlayer::PlayingState)
+        return;
+
+    m_state = QMediaPlayer::PlayingState;
+
     if (m_videoOutput)
         m_videoOutput->setMovie(m_QTMovie);
 
+    AutoReleasePool pool;
     float preferredRate = [[(QTMovie*)m_QTMovie attributeForKey:@"QTMoviePreferredRateAttribute"] floatValue];
-    [(QTMovie*)m_QTMovie setRate:preferredRate*m_rate];
+    [(QTMovie*)m_QTMovie setRate:preferredRate * m_rate];
 
-    if (m_state != QMediaPlayer::PlayingState)
-        emit stateChanged(m_state = QMediaPlayer::PlayingState);
+    emit stateChanged(m_state);
 }
 
 void QT7PlayerSession::pause()
 {
-    if (m_videoOutput)
-        m_videoOutput->setMovie(m_QTMovie);
+    if (m_state == QMediaPlayer::PausedState)
+        return;
 
     m_state = QMediaPlayer::PausedState;
 
+    if (m_videoOutput)
+        m_videoOutput->setMovie(m_QTMovie);
+
     [(QTMovie*)m_QTMovie setRate:0];
 
     emit stateChanged(m_state);
@@ -322,6 +331,9 @@ void QT7PlayerSession::pause()
 
 void QT7PlayerSession::stop()
 {
+    if (m_state == QMediaPlayer::StoppedState)
+        return;
+
     m_state = QMediaPlayer::StoppedState;
 
     [(QTMovie*)m_QTMovie setRate:0];
@@ -330,28 +342,33 @@ void QT7PlayerSession::stop()
     if (m_videoOutput)
         m_videoOutput->setMovie(0);
 
-    if (m_state == QMediaPlayer::StoppedState)
-        emit stateChanged(m_state);
+    emit stateChanged(m_state);
 }
 
 void QT7PlayerSession::setVolume(int volume)
 {
-    if (m_QTMovie) {
-        m_volume = volume;
-        [(QTMovie*)m_QTMovie setVolume:(volume/100.0f)];
-    }
+    if (m_volume == volume)
+        return;
+
+    m_volume = volume;
+
+    if (m_QTMovie != 0)
+        [(QTMovie*)m_QTMovie setVolume:m_volume / 100.0f];
+    else
+        emit volumeChanged(m_volume);
 }
 
 void QT7PlayerSession::setMuted(bool muted)
 {
-    if (m_muted != muted) {
-        m_muted = muted;
+    if (m_muted == muted)
+        return;
 
-        if (m_QTMovie)
-            [(QTMovie*)m_QTMovie setMuted:m_muted];
+    m_muted = muted;
 
-        emit mutedChanged(muted);
-    }
+    if (m_QTMovie != 0)
+        [(QTMovie*)m_QTMovie setMuted:m_muted];
+
+    emit mutedChanged(muted);
 }
 
 QMediaContent QT7PlayerSession::media() const
@@ -368,7 +385,9 @@ void QT7PlayerSession::setMedia(const QMediaContent &content, QIODevice *stream)
 {
     AutoReleasePool pool;
 
-    qDebug() << "SetMedia" << content.canonicalUrl();
+#ifdef QT_DEBUG_QT7
+    qDebug() << Q_FUNC_INFO << content.canonicalUrl();
+#endif
 
     if (m_QTMovie) {
         [(QTMovieObserver*)m_movieObserver setMovie:nil];
@@ -384,12 +403,11 @@ void QT7PlayerSession::setMedia(const QMediaContent &content, QIODevice *stream)
     m_mediaStream = stream;
     m_mediaStatus = QMediaPlayer::NoMedia;
 
-    QNetworkRequest request;
-    if (!content.isNull())
-        request = content.canonicalResource().request();
-    else
+    if (content.isNull())
         return;
 
+    QNetworkRequest request = content.canonicalResource().request();
+
     QVariant cookies = request.header(QNetworkRequest::CookieHeader);
     if (cookies.isValid()) {
         NSHTTPCookieStorage *store = [NSHTTPCookieStorage sharedHTTPCookieStorage];
@@ -412,17 +430,25 @@ void QT7PlayerSession::setMedia(const QMediaContent &content, QIODevice *stream)
         }
     }
 
+    // Attempt multiple times to open the movie.
+    // First try - attempt open in async mode
+    openMovie(true);
+}
 
-    QUrl requestUrl = request.url();
+void QT7PlayerSession::openMovie(bool tryAsync)
+{
+    QUrl requestUrl = m_resources.canonicalResource().request().url();
     if (requestUrl.scheme().isEmpty())
         requestUrl.setScheme(QLatin1String("file"));
 
-    qDebug() << "Play" << requestUrl;
+#ifdef QT_DEBUG_QT7
+    qDebug() << Q_FUNC_INFO << requestUrl;
+#endif
 
     NSError *err = 0;
-    NSString *urlString = qString2CFStringRef(requestUrl.toString());
+    NSString *urlString = [NSString stringWithUTF8String:requestUrl.toEncoded().constData()];
 
-    NSDictionary *attr = [NSDictionary dictionaryWithObjectsAndKeys:
+    NSMutableDictionary *attr = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                 [NSURL URLWithString:urlString], QTMovieURLAttribute,
                 [NSNumber numberWithBool:YES], QTMovieOpenAsyncOKAttribute,
                 [NSNumber numberWithBool:YES], QTMovieIsActiveAttribute,
@@ -430,16 +456,39 @@ void QT7PlayerSession::setMedia(const QMediaContent &content, QIODevice *stream)
                 [NSNumber numberWithBool:YES], QTMovieDontInteractWithUserAttribute,
                 nil];
 
-    m_QTMovie = [[QTMovie movieWithAttributes:attr error:&err] retain];
+    if (tryAsync && QSysInfo::MacintoshVersion >= QSysInfo::MV_10_6) {
+        [attr setObject:[NSNumber numberWithBool:YES] forKey:@"QTMovieOpenAsyncRequiredAttribute"];
+// XXX: This is disabled for now. causes some problems with video playback for some formats
+//        [attr setObject:[NSNumber numberWithBool:YES] forKey:@"QTMovieOpenForPlaybackAttribute"];
+        m_tryingAsync = true;
+    }
+    else
+        m_tryingAsync = false;
 
-    if (err) {
-        [(QTMovie*)m_QTMovie release];
+    m_QTMovie = [QTMovie movieWithAttributes:attr error:&err];
+    if (err != nil) {
+        // First attempt to test for inability to perform async
+//        if ([err code] == QTErrorMovieOpeningCannotBeAsynchronous) { XXX: error code unknown!
+        if (m_tryingAsync) {
+            m_tryingAsync = false;
+            err = nil;
+            [attr removeObjectForKey:@"QTMovieOpenAsyncRequiredAttribute"];
+            m_QTMovie = [QTMovie movieWithAttributes:attr error:&err];
+        }
+    }
+
+    if (err != nil) {
         m_QTMovie = 0;
         QString description = QString::fromUtf8([[err localizedDescription] UTF8String]);
+        emit error(QMediaPlayer::FormatError, description);
+
+#ifdef QT_DEBUG_QT7
+        qDebug() << Q_FUNC_INFO << description;
+#endif
+    }
+    else {
+        [(QTMovie*)m_QTMovie retain];
 
-        qWarning() << "QT7PlayerSession::setMedia error" << description;
-        emit error(QMediaPlayer::FormatError, description );
-    } else {
         [(QTMovieObserver*)m_movieObserver setMovie:(QTMovie*)m_QTMovie];
 
         if (m_state != QMediaPlayer::StoppedState && m_videoOutput)
@@ -448,7 +497,7 @@ void QT7PlayerSession::setMedia(const QMediaContent &content, QIODevice *stream)
         processLoadStateChange();
 
         [(QTMovie*)m_QTMovie setMuted:m_muted];
-        setVolume(m_volume);
+        [(QTMovie*)m_QTMovie setVolume:m_volume / 100.0f];
     }
 }
 
@@ -471,7 +520,10 @@ bool QT7PlayerSession::isVideoAvailable() const
 }
 
 void QT7PlayerSession::processEOS()
-{    
+{
+#ifdef QT_DEBUG_QT7
+    qDebug() << Q_FUNC_INFO;
+#endif
     m_mediaStatus = QMediaPlayer::EndOfMedia;
     if (m_videoOutput)
         m_videoOutput->setMovie(0);
@@ -484,9 +536,13 @@ void QT7PlayerSession::processLoadStateChange()
     if (!m_QTMovie)
         return;
 
-    signed long state = [[(QTMovie*)m_QTMovie attributeForKey:QTMovieLoadStateAttribute]
-                         longValue];
-    qDebug() << "Moview load state changed:" << state;
+    AutoReleasePool pool;
+
+    long state = [[(QTMovie*)m_QTMovie attributeForKey:QTMovieLoadStateAttribute] longValue];
+
+#ifdef QT_DEBUG_QT7
+    qDebug() << Q_FUNC_INFO << state;
+#endif
 
 #ifndef QUICKTIME_C_API_AVAILABLE
     enum {
@@ -499,29 +555,43 @@ void QT7PlayerSession::processLoadStateChange()
     };
 #endif
 
+    if (state == kMovieLoadStateError) {
+        if (m_tryingAsync) {
+            NSError *error = [(QTMovie*)m_QTMovie attributeForKey:@"QTMovieLoadStateErrorAttribute"];
+            if ([error code] == componentNotThreadSafeErr) {
+                // Last Async check, try again with no such flag
+                openMovie(false);
+            }
+        }
+        else {
+            if (m_videoOutput)
+                m_videoOutput->setMovie(0);
+
+            emit error(QMediaPlayer::FormatError, tr("Failed to load media"));
+            emit mediaStatusChanged(m_mediaStatus = QMediaPlayer::InvalidMedia);
+            emit stateChanged(m_state = QMediaPlayer::StoppedState);
+        }
+
+        return;
+    }
+
     QMediaPlayer::MediaStatus newStatus = QMediaPlayer::NoMedia;
     bool isPlaying = (m_state != QMediaPlayer::StoppedState);
 
-    if (state >= kMovieLoadStateComplete) {
+    if (state >= kMovieLoadStatePlaythroughOK) {
         newStatus = isPlaying ? QMediaPlayer::BufferedMedia : QMediaPlayer::LoadedMedia;
     } else if (state >= kMovieLoadStatePlayable)
         newStatus = isPlaying ? QMediaPlayer::BufferingMedia : QMediaPlayer::LoadingMedia;
     else if (state >= kMovieLoadStateLoading)
         newStatus = isPlaying ? QMediaPlayer::StalledMedia : QMediaPlayer::LoadingMedia;
 
-    if (state == kMovieLoadStateError) {
-        newStatus = QMediaPlayer::InvalidMedia;
-        if (m_videoOutput)
-            m_videoOutput->setMovie(0);
-
-        emit error(QMediaPlayer::FormatError, tr("Failed to load media"));
-        emit stateChanged(m_state = QMediaPlayer::StoppedState);
-    }
-
     if (state >= kMovieLoadStatePlayable &&
         m_state == QMediaPlayer::PlayingState &&
         [(QTMovie*)m_QTMovie rate] == 0) {
-            QMetaObject::invokeMethod(this, "play", Qt::QueuedConnection);
+
+        float preferredRate = [[(QTMovie*)m_QTMovie attributeForKey:@"QTMoviePreferredRateAttribute"] floatValue];
+
+        [(QTMovie*)m_QTMovie setRate:preferredRate * m_rate];
     }
 
     if (state >= kMovieLoadStateLoaded) {
@@ -545,7 +615,7 @@ void QT7PlayerSession::processVolumeChange()
     if (!m_QTMovie)
         return;
 
-    int newVolume = qRound(100.0f*[((QTMovie*)m_QTMovie) volume]);
+    int newVolume = qRound(100.0f * [((QTMovie*)m_QTMovie) volume]);
 
     if (newVolume != m_volume) {
         emit volumeChanged(m_volume = newVolume);
@@ -554,8 +624,11 @@ void QT7PlayerSession::processVolumeChange()
 
 void QT7PlayerSession::processNaturalSizeChange()
 {
+    AutoReleasePool pool;
     NSSize size = [[(QTMovie*)m_QTMovie attributeForKey:@"QTMovieNaturalSizeAttribute"] sizeValue];
-    qDebug() << "Native size changed:" << QSize(size.width, size.height);
+#ifdef QT_DEBUG_QT7
+    qDebug() << Q_FUNC_INFO << QSize(size.width, size.height);
+#endif
 
     if (m_videoOutput)
         m_videoOutput->updateNaturalSize(QSize(size.width, size.height));
