@@ -130,7 +130,7 @@ int QAudioInputPrivate::xrun_recovery(int err)
         if(err < 0)
             reset = true;
         else {
-            bytesAvailable = bytesReady();
+            bytesAvailable = checkBytesReady();
             if (bytesAvailable <= 0)
                 reset = true;
         }
@@ -288,7 +288,10 @@ bool QAudioInputPrivate::open()
     QList<QByteArray> devices = QAudioDeviceInfoInternal::availableDevices(QAudio::AudioInput);
     if(dev.compare(QLatin1String("default")) == 0) {
 #if(SND_LIB_MAJOR == 1 && SND_LIB_MINOR == 0 && SND_LIB_SUBMINOR >= 14)
-        dev = QLatin1String(devices.first());
+        if (devices.size() > 0)
+            dev = QLatin1String(devices.first());
+        else
+            return false;
 #else
         dev = QLatin1String("hw:0,0");
 #endif
@@ -429,7 +432,7 @@ bool QAudioInputPrivate::open()
     snd_pcm_start(handle);
 
     // Step 5: Setup timer
-    bytesAvailable = bytesReady();
+    bytesAvailable = checkBytesReady();
 
     if(pullMode)
         connect(audioSource,SIGNAL(readyRead()),this,SLOT(userFeed()));
@@ -458,19 +461,29 @@ void QAudioInputPrivate::close()
     }
 }
 
-int QAudioInputPrivate::bytesReady() const
+int QAudioInputPrivate::checkBytesReady()
 {
     if(resuming)
-        return period_size;
+        bytesAvailable = period_size;
+    else if(deviceState != QAudio::ActiveState
+            && deviceState != QAudio::IdleState)
+        bytesAvailable = 0;
+    else {
+        int frames = snd_pcm_avail_update(handle);
+        if (frames < 0) {
+            bytesAvailable = frames;
+        } else {
+            if((int)frames > (int)buffer_frames)
+                frames = buffer_frames;
+            bytesAvailable = snd_pcm_frames_to_bytes(handle, frames);
+        }
+    }
+    return bytesAvailable;
+}
 
-    if(deviceState != QAudio::ActiveState && deviceState != QAudio::IdleState)
-        return 0;
-    int frames = snd_pcm_avail_update(handle);
-    if (frames < 0) return frames;
-    if((int)frames > (int)buffer_frames)
-        frames = buffer_frames;
-
-    return snd_pcm_frames_to_bytes(handle, frames);
+int QAudioInputPrivate::bytesReady() const
+{
+    return qMax(bytesAvailable, 0);
 }
 
 qint64 QAudioInputPrivate::read(char* data, qint64 len)
@@ -481,12 +494,12 @@ qint64 QAudioInputPrivate::read(char* data, qint64 len)
     if ( !handle )
         return 0;
 
-    bytesAvailable = bytesReady();
+    bytesAvailable = checkBytesReady();
 
     if (bytesAvailable < 0) {
         // bytesAvailable as negative is error code, try to recover from it.
         xrun_recovery(bytesAvailable);
-        bytesAvailable = bytesReady();
+        bytesAvailable = checkBytesReady();
         if (bytesAvailable < 0) {
             // recovery failed must stop and set error.
             close();
@@ -660,11 +673,25 @@ bool QAudioInputPrivate::deviceReady()
         InputPrivate* a = qobject_cast<InputPrivate*>(audioSource);
         a->trigger();
     }
-    bytesAvailable = bytesReady();
+    bytesAvailable = checkBytesReady();
 
     if(deviceState != QAudio::ActiveState)
         return true;
 
+    if (bytesAvailable < 0) {
+        // bytesAvailable as negative is error code, try to recover from it.
+        xrun_recovery(bytesAvailable);
+        bytesAvailable = checkBytesReady();
+        if (bytesAvailable < 0) {
+            // recovery failed must stop and set error.
+            close();
+            errorState = QAudio::IOError;
+            deviceState = QAudio::StoppedState;
+            emit stateChanged(deviceState);
+            return 0;
+        }
+    }
+
     if(intervalTime && (timeStamp.elapsed() + elapsedTimeOffset) > intervalTime) {
         emit notify();
         elapsedTimeOffset = timeStamp.elapsed() + elapsedTimeOffset - intervalTime;
