@@ -644,7 +644,7 @@ private:
     imageCapture->setImageSettings(imageSettings);
     \endcode
 
-    \sa QStillImageCapture, QImageEncoderControl
+    \sa QImageEncoderControl
 */
 
 /*!
