@@ -99,7 +99,7 @@ QT_BEGIN_NAMESPACE
         player->setPlaylist(playlist);
 
         widget = new QVideoWidget;
-        player->addVideoOutput(widget);
+        player->setVideoOutput(widget);
         widget->show();
 
         player->play();
@@ -685,12 +685,12 @@ QStringList QMediaPlayer::supportedMimeTypes(Flags flags)
 }
 
 /*!
+    \fn void QMediaPlayer::setVideoOutput(QVideoWidget* output)
+
     Attach a QVideoWidget video \a output to the media player.
 
     If the media player has already video output attached,
     it will be replaced with a new one.
-
-    \sa setVideoOutput(QGraphicsVideoItem*)
 */
 void QMediaPlayer::setVideoOutput(QVideoWidget *output)
 {
@@ -706,12 +706,12 @@ void QMediaPlayer::setVideoOutput(QVideoWidget *output)
 }
 
 /*!
+    \fn void QMediaPlayer::setVideoOutput(QGraphicsVideoItem* output)
+
     Attach a QGraphicsVideoItem video \a output to the media player.
 
     If the media player has already video output attached,
     it will be replaced with a new one.
-
-    \sa setVideoOutput(QVideoWidget*)
 */
 void QMediaPlayer::setVideoOutput(QGraphicsVideoItem *output)
 {
