@@ -51,9 +51,15 @@
 #include <QDir>
 #include <QRegExp>
 #include <QTimer>
+#include <QList>
 
 #include <sysutil.h>
+#ifdef HB_SUPPORTED
+#include <hbinputkeymapfactory.h>
+#include <hbinputlanguage.h>
+#else
 #include <ptiengine.h>
+#endif // HB_SUPPORTED
 #include <featdiscovery.h>
 #ifndef KFeatureIdMmc
 #include <featureinfo.h>
@@ -70,6 +76,8 @@
 #include <bttypes.h>
 #include <etel3rdparty.h>
 #include <aknkeylock.h>
+#define RD_STARTUP_CHANGE
+#include <startupdomainpskeys.h>
 
 QTM_BEGIN_NAMESPACE
 
@@ -87,6 +95,21 @@ QString QSystemInfoPrivate::currentLanguage() const
     return TLanguageToISO639_1(User::Language());
 }
 
+#ifdef HB_SUPPORTED
+QStringList QSystemInfoPrivate::availableLanguages() const
+{
+    QStringList languages;
+    QList<HbInputLanguage> hblanguages = HbKeymapFactory::availableLanguages();
+    foreach(HbInputLanguage lang, hblanguages) {       
+        QString language = QLocaleToISO639_1(lang.language());
+        if (!language.isEmpty()) {
+            languages << language;
+        }
+    }
+    languages.removeDuplicates();
+    return languages;
+}  
+#else
 QStringList QSystemInfoPrivate::availableLanguages() const
 {
     QStringList languages;
@@ -107,6 +130,60 @@ QStringList QSystemInfoPrivate::availableLanguages() const
     languages.removeDuplicates();
     return languages;
 }
+#endif //HB_SUPPORTED
+
+QString QSystemInfoPrivate::QLocaleToISO639_1(QLocale::Language language) const    
+{
+       switch(language) {
+       case QLocale::English: return "en";
+       case QLocale::Lithuanian: return "lt";
+       case QLocale::Malay: return "ms";
+       case QLocale::Polish: return "pl";
+       case QLocale::Portuguese: return "pt";
+       case QLocale::Romanian: return "ro";
+       case QLocale::Serbian: return "sr";
+       case QLocale::Slovak: return "sk";
+       case QLocale::Slovenian: return "sl";
+       case QLocale::Spanish: return "es";
+       case QLocale::Swedish: return "sv";
+       case QLocale::Tagalog: return "tl";
+       case QLocale::Czech: return "cs";
+       case QLocale::Dutch: return "nl";
+       case QLocale::Turkish: return "tr";
+       case QLocale::Estonian: return "et";
+       case QLocale::French: return "fr";
+       case QLocale::Greek: return "el";
+       case QLocale::Icelandic: return "is";
+       case QLocale::Indonesian: return "id";
+       case QLocale::Italian: return "it";
+       case QLocale::Latvian: return "lv";
+       case QLocale::Croatian: return "hr";
+       case QLocale::German: return "de";
+       case QLocale::Hungarian: return "hu";
+       case QLocale::Bulgarian: return "bg";
+       case QLocale::Finnish: return "fi";
+       case QLocale::Russian: return "ru";
+       case QLocale::Danish: return "da";
+       case QLocale::Norwegian: return "no";
+       case QLocale::Ukrainian: return "uk";
+       case QLocale::Arabic: return "ar";
+       case QLocale::Hebrew: return "he";
+       case QLocale::Thai: return "th";
+       case QLocale::Japanese: return "ja";
+       case QLocale::Vietnamese: return "vi";
+       case QLocale::Persian: return "fa";
+       case QLocale::Hindi: return "hi";
+       case QLocale::Urdu: return "ur";
+       case QLocale::Catalan: return "ca";
+       case QLocale::Galician: return "gl";
+       case QLocale::Basque: return "eu";
+       case QLocale::Marathi: return "mr";
+       case QLocale::Korean: return "ko";       
+       default:
+           break;
+       }
+    return "";        
+}
 
 QString QSystemInfoPrivate::TLanguageToISO639_1(TLanguage language) const
 {
@@ -312,6 +389,12 @@ QSystemNetworkInfoPrivate::QSystemNetworkInfoPrivate(QObject *parent)
     DeviceInfo::instance()->cellSignalStrenghtInfo()->addObserver(this);
     DeviceInfo::instance()->cellNetworkInfo()->addObserver(this);
     DeviceInfo::instance()->cellNetworkRegistrationInfo()->addObserver(this);
+    connect(DeviceInfo::instance()->wlanInfo(), SIGNAL(wlanNetworkNameChanged()),
+        this, SLOT(wlanNetworkNameChanged()));
+    connect(DeviceInfo::instance()->wlanInfo(), SIGNAL(wlanNetworkSignalStrengthChanged()),
+        this, SLOT(wlanNetworkSignalStrengthChanged()));
+    connect(DeviceInfo::instance()->wlanInfo(), SIGNAL(wlanNetworkStatusChanged()),
+        this, SLOT(wlanNetworkStatusChanged()));
 }
 
 QSystemNetworkInfoPrivate::~QSystemNetworkInfoPrivate()
@@ -356,6 +439,12 @@ QSystemNetworkInfo::NetworkStatus QSystemNetworkInfoPrivate::networkStatus(QSyst
             };
         }
         case QSystemNetworkInfo::WlanMode:
+        {
+            if (DeviceInfo::instance()->wlanInfo()->wlanNetworkConnectionStatus())
+                return QSystemNetworkInfo::Connected;
+            else
+                return QSystemNetworkInfo::NoNetworkAvailable;
+        }
         case QSystemNetworkInfo::EthernetMode:
         case QSystemNetworkInfo::BluetoothMode:
         case QSystemNetworkInfo::WimaxMode:
@@ -387,6 +476,7 @@ int QSystemNetworkInfoPrivate::networkSignalStrength(QSystemNetworkInfo::Network
         }
 
         case QSystemNetworkInfo::WlanMode:
+            return DeviceInfo::instance()->wlanInfo()->wlanNetworkSignalStrength();
         case QSystemNetworkInfo::EthernetMode:
         case QSystemNetworkInfo::BluetoothMode:
         case QSystemNetworkInfo::WimaxMode:
@@ -457,6 +547,7 @@ QString QSystemNetworkInfoPrivate::networkName(QSystemNetworkInfo::NetworkMode m
             return DeviceInfo::instance()->cellNetworkInfo()->networkName();
         }
         case QSystemNetworkInfo::WlanMode:
+            return DeviceInfo::instance()->wlanInfo()->wlanNetworkName();
         case QSystemNetworkInfo::EthernetMode:
         case QSystemNetworkInfo::BluetoothMode:
         case QSystemNetworkInfo::WimaxMode:
@@ -545,6 +636,28 @@ void QSystemNetworkInfoPrivate::cellNetworkStatusChanged()
     emit networkStatusChanged(mode, networkStatus(mode));
 }
 
+void QSystemNetworkInfoPrivate::wlanNetworkNameChanged()
+{
+    emit networkNameChanged(QSystemNetworkInfo::WlanMode,
+        DeviceInfo::instance()->wlanInfo()->wlanNetworkName());
+}
+
+void QSystemNetworkInfoPrivate::wlanNetworkSignalStrengthChanged()
+{
+    emit networkSignalStrengthChanged(QSystemNetworkInfo::WlanMode,
+        DeviceInfo::instance()->wlanInfo()->wlanNetworkSignalStrength());
+}
+
+//TODO: There are no WLAN specific modes (Not connected, Infrastructure, Adhoc, Secure Infrastructure and Searching)
+void QSystemNetworkInfoPrivate::wlanNetworkStatusChanged()
+{
+    bool status = DeviceInfo::instance()->wlanInfo()->wlanNetworkConnectionStatus();
+    if (status)
+        emit networkStatusChanged(QSystemNetworkInfo::WlanMode, QSystemNetworkInfo::Connected);
+    else
+        emit networkStatusChanged(QSystemNetworkInfo::WlanMode, QSystemNetworkInfo::NoNetworkAvailable);
+}
+
 QSystemNetworkInfo::NetworkMode QSystemNetworkInfoPrivate::currentMode()
 {
     QSystemNetworkInfo::NetworkMode mode = QSystemNetworkInfo::UnknownMode;
@@ -869,7 +982,10 @@ QString QSystemDeviceInfoPrivate::imei()
 
 QString QSystemDeviceInfoPrivate::imsi()
 {
-    return DeviceInfo::instance()->subscriberInfo()->imsi();
+    if (simStatus() == QSystemDeviceInfo::SimNotAvailable)
+        return QString();
+    else
+        return DeviceInfo::instance()->subscriberInfo()->imsi();
 }
 
 QString QSystemDeviceInfoPrivate::manufacturer()
@@ -915,7 +1031,19 @@ QSystemDeviceInfo::BatteryStatus QSystemDeviceInfoPrivate::batteryStatus()
 
 QSystemDeviceInfo::SimStatus QSystemDeviceInfoPrivate::simStatus()
 {
-    return QSystemDeviceInfo::SimNotAvailable;  //Not available in public SDKs
+    TInt lockStatus = 0;
+    TInt err = RProperty::Get(KPSUidStartup, KStartupSimLockStatus, lockStatus);
+    if (err == KErrNone && (TPSSimLockStatus)lockStatus != ESimLockOk) {
+        return QSystemDeviceInfo::SimLocked;
+    }
+
+    TInt simStatus = 0;
+    err = RProperty::Get(KPSUidStartup, KPSSimStatus, simStatus);
+    if (err == KErrNone && TPSSimStatus(simStatus) == ESimUsable) {
+        return QSystemDeviceInfo::SingleSimAvailable;
+    }
+
+    return QSystemDeviceInfo::SimNotAvailable;
 }
 
 bool QSystemDeviceInfoPrivate::isDeviceLocked()
